////////////////////////////////////
// clock AND protoThreads configure!
// You MUST check this file!
#include "config.h"
// threading library
#include "pt_cornell_1_1.h"
#include <math.h>

// need for rand function
#include <stdlib.h>
////////////////////////////////////

char buffer[60];

static struct pt pt_spi, pt_fcn, pt_oscilliscope;

#define DAC_config_chan_A 0b0011000000000000
#define Fs 25000.0
#define two32 4294967296.0 // 2^32 

// === 16:16 fixed point macros ==========================================
typedef signed int fix16 ;
#define multfix16(a,b) ((fix16)(((( signed long long)(a))*(( signed long long)(b)))>>16)) //multiply two fixed 16:16
#define float2fix16(a) ((fix16)((a)*65536.0)) // 2^16
#define fix2float16(a) ((float)(a)/65536.0)
#define fix2int16(a)    ((int)((a)>>16))
#define int2fix16(a)    ((fix16)((a)<<16))
#define divfix16(a,b) ((fix16)((((signed long long)(a)<<16)/(b)))) 
#define sqrtfix16(a) (float2fix16(sqrt(fix2float16(a)))) 
#define absfix16(a) abs(a)
#define onefix16 0x00010000 // int2fix16(1)

#define float2fix7(a) ((fix16)((a)*128.0)) // 2^16
#define fix2float7(a) ((float)(a)/128.0)

// DDS sine table
#define table_size 256

// for 60 MHz PB clock use divide-by-3
int freq1 = 50;
int wType = 0;
int offset = 2048;
float amplitude = .5;
int spiClkDiv = 2 ; // 15 MHz DAC clock
fix16 cos_table[table_size] ;
fix16 square_table[table_size] ;
fix16 saw_table[table_size] ;

//== Timer 2 interrupt handler ===========================================
// actual scaled DAC 
volatile int DAC_data;
// the DDS units: 1477 
volatile unsigned int phase_accum_main, phase_incr_main_const=two32/Fs, phase_incr_main;
// profiling of ISR
volatile int isr_time;
// when and what to output
volatile unsigned int freq;

void Timer2Handler(void);

// === thread structures ============================================
// thread control structs
// note that UART input and output are threads
volatile int adc_9;
volatile float voltage;
volatile unsigned int data;
volatile int amplitudeData;
volatile int sixteenVoltage;

const int samples = 600;
volatile int adcBuffer[600];
volatile int counter = 0;
volatile int sent = 0;

void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void)
{
    mT2ClearIntFlag();
    adc_9 = ReadADC10(0);   // read the result of channel 9 conversion from the idle buffer
    //AcquireADC10(); // not needed if ADC_AUTO_SAMPLING_ON below
    
    if (counter < samples){
        adcBuffer[counter] = adc_9;
        counter++;
    }
    
    phase_incr_main = freq1 * phase_incr_main_const;

    phase_accum_main += phase_incr_main;
    if (wType == 0){
        DAC_data = 2.0 * amplitude * cos_table[phase_accum_main>>24];  
    } else if (wType == 1){
        DAC_data = 2.0 * amplitude * square_table[phase_accum_main>>24];
    } else {
        DAC_data = 2.0 * amplitude * saw_table[phase_accum_main>>24];
    }

    // CS low to start transaction for dual tone DDS
    mPORTBClearBits(BIT_4); // start transaction
    WriteSPI1( DAC_config_chan_A | (DAC_data + offset));
    while(SPI1STATbits.SPIBUSY);
    // end SPI transaction from last interrupt cycle
    mPORTBSetBits(BIT_4); 
} // end ISR TIMER2

#pragma config DEBUG = OFF

// this debounce state machine must call the scanner pt_key over and overW
static PT_THREAD (protothread_fcn(struct pt *pt)) {
    PT_BEGIN(pt);
    
    while(1) {
        freq = 1000;
        PT_YIELD_TIME_msec(1000);
    } // end while
      
    PT_END(pt);
} //end thread

static unsigned int txferSize;
static unsigned short slaveBuff[5];
static PT_THREAD (protothread_spi (struct pt *pt)) {
    PT_BEGIN(pt);
  
    
    while(1){

        
        //PT_YIELD_TIME_msec(1);
    }
    
    PT_END(pt);
}

int main (){
    //SYSTEMConfigPerformance(PBCLK);
    ANSELA = 0; ANSELB = 0; CM1CON = 0; CM2CON = 0;

    // === config threads ==========
    // turns OFF UART support and debugger pin, unless defines are set
    PT_setup();

    // === setup system wide interrupts  ========
    INTEnableSystemMultiVectoredInt();
    //40 MHZ/ 25 KHz = 1600
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 1600);
     // set up the timer interrupt with a priority of 2
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    
    ///DAC SPI setup //////////////////////////////////////////
    // SCK1 is pin 25 
    // SDO1 (MOSI) is in PPS output group 2, could be connected to RB5 which is pin 14
    //mPORTBSetPinsDigitalOut(BIT_15);
    PPSOutput(2, RPB5, SDO1);
    // control CS for DAC
    mPORTBSetPinsDigitalOut(BIT_4);
    
    // divide Fpb by BIT_02, configure the I/O ports. Not using SS in this example
    // 16 bit transfer CKP=1 CKE=1
    // possibles SPI_OPEN_CKP_HIGH;   SPI_OPEN_SMP_END;  SPI_OPEN_CKE_REV
    // For any given peripheral, you will need to match these
    SpiChnOpen( SPI_CHANNEL1, SPI_OPEN_ON | SPI_OPEN_MODE16 | SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV , spiClkDiv);
    
    // build the sine lookup table
    // scaled to produce values between 0 and 4096
    int i;
    for (i = 0; i < table_size; i++){
          cos_table[i] = (int)(1023*cos((float)i*6.283/(float)table_size));
    }
    
    for (i = 0; i < table_size; i++){
        if (i < table_size/2){
            square_table[i] = -1023;
        }  else {
            square_table[i] = 1023;
        }
    }
    
    for (i = 0; i < table_size; i++){
        if (i < table_size/2){
            //saw_table[i] = (int)(i * (1023 / ((table_size - 1)/2)));
            saw_table[i] = -1023 + i * ((float) 2046.0/( (((float)table_size)/2) - 1));
        }  else {
            //saw_table[i] = (int)(1023 - (i - table_size) * (1023 / (table_size - 1)));
            saw_table[i] = (int) (1023 - (i-128) * ((float) 2046.0/((float)(table_size/2 - 1))));
        }
    }
    
    // Setup SPI Communication
    OpenSPI2( SPI_MODE32_ON | SLAVE_ENABLE_ON | SPI_CKE_ON | ENABLE_SDO_PIN | MASTER_ENABLE_OFF | CLK_POL_ACTIVE_HIGH, SPI_ENABLE);
    PPSInput(4, SS2, RPA3); // Pin 10
    PPSInput(3, SDI2, RPA4);// Pin 9
    PPSOutput(2, RPA1, SDO2); // pin 3

    // the ADC ///////////////////////////////////////
    // configure and enable the ADC
	CloseADC10();	// ensure the ADC is off before setting the configuration

	// define setup parameters for OpenADC10
	// Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON //

	// define setup parameters for OpenADC10
	// ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off
	#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

	// Define setup parameters for OpenADC10
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    // ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_0 | ADC_CONV_CLK_Tcy2 //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy2

	// define setup parameters for OpenADC10
	// set AN4 and  as analog inputs
	#define PARAM4	ENABLE_AN11_ANA // pin 24

	// define setup parameters for OpenADC10
	// do not assign channels to scan
	#define PARAM5	SKIP_SCAN_ALL

	// use ground as neg ref for A | use AN4 for input A     
	// configure to sample AN11 
	SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN11 ); // configure to sample AN4 
	OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using the parameters defined above

	EnableADC10(); // Enable the ADC
    
    
#ifdef TFT
    // init the display
    tft_init_hw();
    tft_begin();
    tft_fillScreen(ILI9340_BLACK);
    //240x320 vertical display
    tft_setRotation(0); // Use tft_setRotation(1) for 320x240
    tft_setCursor(0, 0);
    tft_setTextColor(ILI9340_WHITE);  tft_setTextSize(1);
#endif
    
    // === now the threads ==================================
    // init the threads
    //PT_INIT(&pt_spi);
    //PT_INIT(&pt_fcn);
    
    while (1){
        //PT_SCHEDULE(protothread_fcn(&pt_fcn));
        //PT_SCHEDULE(protothread_spi(&pt_spi));

        
        if (counter >= samples - 1){
            putcSPI2(0xFFFF0000); 
            
            if (DataRdySPI2()){
                //INTDisableInterrupts();
                data = ReadSPI2();
                if (data > 0){
                    freq1 = data & 0xFFFF;
                    amplitudeData = (data >> 16) & 0xFF;
                    amplitude = fix2float7(amplitudeData);
                    wType = (data >> 24) & 0x3;
                }
                //INTEnableSystemMultiVectoredInt();
            }
            int i;
            for (i = 0; i < samples; i++){
                voltage = (adcBuffer[i] / 1008.0) * 3.3;
                sixteenVoltage = float2fix16(voltage);
                putcSPI2(sixteenVoltage);    
                
                if (DataRdySPI2()){
                    //INTDisableInterrupts();
                    data = ReadSPI2();
                    if (data > 0){
                        freq1 = data & 0xFFFF;
                        amplitudeData = (data >> 16) & 0xFF;
                        amplitude = fix2float7(amplitudeData);
                        wType = (data >> 24) & 0x3;
                    }
                    //INTEnableSystemMultiVectoredInt();
                }
            }
            counter = 0;
        } else {
            putcSPI2(0x0000);    
                
            if (DataRdySPI2()){
                //INTDisableInterrupts();
                data = ReadSPI2();
                if (data > 0){
                    freq1 = data & 0xFFFF;
                    amplitudeData = (data >> 16) & 0xFF;
                    amplitude = fix2float7(amplitudeData);
                    wType = (data >> 24) & 0x3;
                }
                //INTEnableSystemMultiVectoredInt();
            }
        }
    

    } // main
}


